﻿<?php

Class BdD {
    /**
     * Regroupe les fonctions d'accès aux données.
     * @package default
     * @author Arthur Martin, modifié par Alain Arsane en Septembre 2018
     */

    /**
     * Se connecte au serveur de données MySql.                      
     * Se connecte au serveur de données MySql à partir de valeurs
     * prédéfinies de connexion (hôte, compte utilisateur et mot de passe). 
     * Retourne l'identifiant de connexion si succès obtenu, le booléen false 
     * si problème de connexion.
     * @return resource identifiant de connexion
     */
    public static function connecterServeurBD() {
        $PARAM_hote = 'localhost'; // le chemin vers le serveur
        $PARAM_nom_bd = 'gsb_frais'; // le nom de votre base de données
        $PARAM_utilisateur = 'userGsb'; // nom d'utilisateur pour se connecter
        $PARAM_mot_passe = 'secret'; // mot de passe de l'utilisateur pour se connecter
        $_connexion = null;
        try {
            $_connexion = new PDO('mysql:host=' . $PARAM_hote . ';dbname=' . $PARAM_nom_bd, $PARAM_utilisateur, $PARAM_mot_passe);
            $_connexion->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
            return $_connexion;
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Fournit les informations sur un visiteur demandé. 
     * Retourne les informations du visiteur d'id $unId sous la forme d'un tableau
     * associatif dont les clés sont les noms des colonnes(id, nom, prenom).
     * @param string $unId id de l'utilisateur
     * @return array  tableau associatif du visiteur
     */
    public static function obtenirDetailVisiteur($unId) {
        try {
            $requete = "select id, nom, prenom from Visiteur where id= ?";
            $_connexion = BdD::connecterServeurBD();
            $ligne = false;
            if ($_connexion) {
                $prep = $_connexion->prepare($requete);
                $prep->execute(array($unId));

                if ($prep) {
                    $ligne = $prep->fetch(PDO::FETCH_ASSOC);
                }
                $_connexion = null; // On ferme la connexion  
            }
            return $ligne;
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Fournit les informations d'une fiche de frais. 
     * Retourne les informations de la fiche de frais du mois de $unMois (MMAAAA)
     * sous la forme d'un tableau associatif dont les clés sont les noms des colonnes
     * (nbJustitificatifs, idEtat, libelleEtat, dateModif, montantValide).
     * @param string $unMois mois demandé (MMAAAA)
     * @param string $unIdVisiteur id visiteur  
     * @return array tableau associatif de la fiche de frais
     */
    public static function obtenirDetailFicheFrais($unMois, $unIdVisiteur) {
        try {
            $ligne = false;
            $requete = "select IFNULL(nbJustificatifs,0) as nbJustificatifs, Etat.id as idEtat, libelle as libelleEtat, dateModif, montantValide 
            from FicheFrais inner join Etat on idEtat = Etat.id 
            where idVisiteur= ? and mois= ?";
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                $prep = $_connexion->prepare($requete);
                $prep->execute(array($unIdVisiteur, $unMois));
                $ligne = false;
                if ($prep) {
                    $ligne = $prep->fetch(PDO::FETCH_ASSOC);
                }
                $_connexion = null; // On ferme la connexion  
            }
            return $ligne;
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Vérifie si une fiche de frais existe ou non. 
     * Retourne true si la fiche de frais du mois de $unMois (MMAAAA) du visiteur 
     * $idVisiteur existe, false sinon. 
     * @param string $unMois mois demandé (MMAAAA)
     * @param string $unIdVisiteur id visiteur  
     * @return booléen existence ou non de la fiche de frais
     */
    public static function existeFicheFrais($unMois, $unIdVisiteur) {
        try {
            $requete = "select idVisiteur from FicheFrais where idVisiteur=? and mois=?";
            $ligne = false;
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                $prep = $_connexion->prepare($requete);
                $prep->execute(array($unIdVisiteur, $unMois));
                if ($prep) {
                    $ligne = $prep->fetch(PDO::FETCH_ASSOC);
                }
                $_connexion = null; // On ferme la connexion  
            }
            // si $ligne est un tableau, la fiche de frais existe, sinon elle n'exsite pas
            return is_array($ligne);
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Fournit le mois de la dernière fiche de frais d'un visiteur.
     * Retourne le mois de la dernière fiche de frais du visiteur d'id $unIdVisiteur.
     * @param string $unIdVisiteur id visiteur  
     * @return string dernier mois sous la forme AAAAMM
     */
    public static function obtenirDernierMoisSaisi($unIdVisiteur) {
        try {
            $requete = "select max(mois) as dernierMois from FicheFrais where idVisiteur=?";
            $dernierMois = false;
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                $prep = $_connexion->prepare($requete);
                $prep->execute(array($unIdVisiteur));

                if ($prep) {
                    $ligne = $prep->fetch(PDO::FETCH_ASSOC);
                    $dernierMois = $ligne["dernierMois"];
                }
                $_connexion = null; // On ferme la connexion  
            }
            return $dernierMois;
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Ajoute une nouvelle fiche de frais et les éléments forfaitisés associés, 
     * Ajoute la fiche de frais du mois de $unMois (MMAAAA) du visiteur 
     * $idVisiteur, avec les éléments forfaitisés associés dont la quantité initiale
     * est affectée é 0. Clét éventuellement la fiche de frais précédente du visiteur. 
     * @param string $unMois mois demandé (MMAAAA)
     * @param string $unIdVisiteur id visiteur  
     * @return void
     */
    public static function ajouterFicheFrais($unMois, $unIdVisiteur) {
        try {
            $dernierMois = BdD::obtenirDernierMoisSaisi($unIdVisiteur);
            $laDerniereFiche = BdD::obtenirDetailFicheFrais($dernierMois, $unIdVisiteur);
            if (is_array($laDerniereFiche) && $laDerniereFiche['idEtat'] == 'CR') {
                BdD::modifierEtatFicheFrais($dernierMois, $unIdVisiteur, 'CL');
            }
            // ajout de la fiche de frais à l'état Créé
            BdD::insererFicheFraisEtatCR($unMois, $unIdVisiteur);

            // ajout des éléments forfaitisés
            BdD::insererLignesFraisForfait($unMois, $unIdVisiteur);
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Insère une FichFrais à l'état CR
     * @param type $unMois : mois de la fiche
     * @param type $unIdVisiteur : id du visiteur concerné
     * @return void
     */
    public static function insererLignesFraisForfait($unMois, $unIdVisiteur) {
        try {
            $requete = "select id from FraisForfait";
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                $prep = $_connexion->prepare($requete);
                $prep->execute();
                if ($prep) {
                    $lignes = $prep->fetchAll(PDO::FETCH_ASSOC);
                }
                foreach ($lignes as $ligne) {
                    $idFraisForfait = $ligne["id"];
                    // insertion d'une ligne frais forfait dans la base
                    $requete = "insert into LigneFraisForfait (idVisiteur, mois, idFraisForfait, quantite)
                values (?,?,?,0)";
                    $prep = $_connexion->prepare($requete);
                    $prep->execute(array($unIdVisiteur, $unMois, $idFraisForfait));
                }
                $_connexion = null;
            }
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Ajout de la fiche de frais à l'état Créé
     * @param type $unMois : mois de la fiche
     * @param type $unIdVisiteur : id du visiteur concerné
     * @return void
     */
    public static function insererFicheFraisEtatCR($unMois, $unIdVisiteur) {
        try {
            $dateJour = date("Y-m-d");
            $requete = "insert into FicheFrais (idVisiteur, mois, nbJustificatifs, montantValide, idEtat, dateModif)"
                    . " values (? ,?,0,NULL, 'CR', ?)";
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                $prep = $_connexion->prepare($requete);
                $prep->execute(array($unIdVisiteur, $unMois, $dateJour));
                $_connexion = null;
            }
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Retourne le texte de la requête select concernant les mois pour lesquels un 
     * visiteur a une fiche de frais. 
     * La requête de sélection fournie permettra d'obtenir les mois (AAAAMM) pour 
     * lesquels le visiteur $unIdVisiteur a une fiche de frais. 
     * @param string $unIdVisiteur id visiteur  
     * @return string texte de la requête select
     */
    public static function obtenirReqMoisFicheFrais($unIdVisiteur) {
        try {
            $requete = "select mois from  FicheFrais where idvisiteur = ? order by mois desc";
            $lignes = false;
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                $prep = $_connexion->prepare($requete);
                $prep->execute(array($unIdVisiteur));
                if ($prep) {
                    $lignes = $prep->fetchAll(PDO::FETCH_ASSOC);
                }
                $_connexion = null; // On ferme la connexion 
            }
            return $lignes;
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
        return $req;
    }

    /**
     * Retourne le texte de la requête select concernant les éléments forfaitisés 
     * d'un visiteur pour un mois donnés. 
     * La requête de sélection fournie permettra d'obtenir l'id, le libellé et la
     * quantité des éléments forfaitisés de la fiche de frais du visiteur
     * d'id $idVisiteur pour le mois $mois    
     * @param string $unMois mois demandé (MMAAAA)
     * @param string $unIdVisiteur id visiteur  
     * @return tableau de lignes
     */
    public static function obtenirReqEltsForfaitFicheFrais($unMois, $unIdVisiteur) {
        try {
            $requete = "select idFraisForfait, libelle, quantite from LigneFraisForfait
              inner join FraisForfait on FraisForfait.id = LigneFraisForfait.idFraisForfait
              where idVisiteur=? and mois=?";
            $lignes = false;
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                $prep = $_connexion->prepare($requete);
                $prep->execute(array($unIdVisiteur, $unMois));
                if ($prep) {
                    $lignes = $prep->fetchAll(PDO::FETCH_ASSOC);
                }
                $_connexion = null; // On ferme la connexion  
            }
            return $lignes;
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Retourne un tableau de lignes hors forfait 
     * d'un visiteur pour un mois donné. 
     * 
     * La requête de sélection fournie permettra d'obtenir l'id, la date, le libellé 
     * et le montant des éléments hors forfait de la fiche de frais du visiteur
     * d'id $idVisiteur pour le mois $mois    
     * @param string $unMois mois demandé (MMAAAA)
     * @param string $unIdVisiteur id visiteur  
     * @return tableau de lignes hors forfait
     */
    public static function obtenirReqEltsHorsForfaitFicheFrais($unMois, $unIdVisiteur) {
        try {
            $requete = "select id, date, libelle, montant from LigneFraisHorsForfait
              where idVisiteur=? and mois=?";
            $lignes = false;
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                $prep = $_connexion->prepare($requete);
                $prep->execute(array($unIdVisiteur, $unMois));
                if ($prep) {
                    $lignes = $prep->fetchAll(PDO::FETCH_ASSOC);
                }
                $_connexion = null; // On ferme la connexion  
            }
            return $lignes;
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Supprime une ligne hors forfait.
     * Supprime dans la BD la ligne hors forfait d'id $unIdLigneHF
     * @param string $unIdLigneHF id de la ligne hors forfait
     * @return void
     */
    public static function supprimerLigneHF($unIdLigneHF) {
        try {
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                $requete = "delete from LigneFraisHorsForfait where id = ?";
                $prep = $_connexion->prepare($requete);
                $prep->execute(array($unIdLigneHF));
                $_connexion = null;
            }
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Ajoute une nouvelle ligne hors forfait.
     * Insére dans la BD la ligne hors forfait de libellé $unLibelleHF du montant 
     * $unMontantHF ayant eu lieu à la date $uneDateHF pour la fiche de frais du mois
     * $unMois du visiteur d'id $unIdVisiteur
     * @param string $unMois mois demandé (AAMMMM)
     * @param string $unIdVisiteur id du visiteur
     * @param string $uneDateHF date du frais hors forfait
     * @param string $unLibelleHF libellé du frais hors forfait 
     * @param double $unMontantHF montant du frais hors forfait
     * @return void
     */
    public static function ajouterLigneHF($unMois, $unIdVisiteur, $uneDateHF, $unLibelleHF, $unMontantHF) {
        try {
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                $uneDateHF = convertirDateFrancaisVersAnglais($uneDateHF);
                $requete = "insert into LigneFraisHorsForfait(idVisiteur, mois, date, libelle, montant) 
                values (?,?,?,?,?)";
                $prep = $_connexion->prepare($requete);
                $prep->execute(array($unIdVisiteur, $unMois, $uneDateHF, $unLibelleHF, $unMontantHF));
                $_connexion = null;
            }
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Modifie les quantités des éléments forfaitisés d'une fiche de frais. 
     * Met à jour les éléments forfaitisés contenus  
     * dans $desEltsForfaits pour le visiteur $unIdVisiteur et
     * le mois $unMois dans la table LigneFraisForfait
     * @param string $unMois mois demandé (MMAAAA) 
     * @param string $unIdVisiteur  id visiteur
     * @param array $desEltsForfait tableau des quantités des éléments hors forfait
     * avec pour clés les identifiants des frais forfaitisés 
     * @return void  
     */
    public static function modifierEltsForfait($unMois, $unIdVisiteur, $desEltsForfait) {
        try {
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                foreach ($desEltsForfait as $idFraisForfait => $quantite) {
                    $requete = "update LigneFraisForfait set quantite =? "
                            . " where idVisiteur = ? and mois = ?"
                            . " and idFraisForfait= ?";
                    $prep = $_connexion->prepare($requete);
                    $prep->execute(array($quantite, $unIdVisiteur, $unMois, $idFraisForfait));
                }
                $_connexion = null;
            }
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Contrôle les informations de connexionn d'un utilisateur.
     * Vérifie si les informations de connexion $unLogin, $unMdp sont ou non valides.
     * Retourne les informations de l'utilisateur sous forme de tableau associatif 
     * dont les clés sont les noms des colonnes (id, nom, prenom, login, mdp)
     * si login et mot de passe existent, le booléen false sinon. 
     * @param string $unLogin login 
     * @param string $unMdp mot de passe 
     * @return array tableau associatif ou booléen false 
     */
    public static function verifierInfosConnexion($unLogin, $unMdp) {
        try {
            // le mot de passe est crypté dans la base avec la fonction de hachage md5
            $req = "select id, nom, prenom, login, mdp from Visiteur where login=? and mdp=?";
            $_connexion = BdD::connecterServeurBD();
            $ligne = false;
            if ($_connexion) {
                $prep = $_connexion->prepare($req);
                $prep->execute(array($unLogin, $unMdp));
                if ($prep) {
                    $ligne = $prep->fetch(PDO::FETCH_ASSOC);
                }
                $_connexion = null; // On ferme la connexion 
            }
            return $ligne;
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

    /**
     * Modifie l'état et la date de modification d'une fiche de frais

     * Met à jour l'état de la fiche de frais du visiteur $unIdVisiteur pour
     * le mois $unMois à la nouvelle valeur $unEtat et passe la date de modif à 
     * la date d'aujourd'hui
     * @param string $unMois mois sous la forme aaaamm
     * @param string $unIdVisiteur 
     * @param string $unEtat état à écrire
     * @return void 
     */
    public static function modifierEtatFicheFrais($unMois, $unIdVisiteur, $unEtat) {
        try {
            $_connexion = BdD::connecterServeurBD();
            if ($_connexion) {
                $uneDateHF = convertirDateFrancaisVersAnglais($uneDateHF);
                $requete = "update FicheFrais set idEtat = ?, dateModif = now() where idVisiteur =? and mois = ?";
                $prep = $_connexion->prepare($requete);
                $prep->execute(array($unEtat, $uneDateHF, $unIdVisiteur, $unMois));
                $_connexion = null;
            }
        } catch (Exception $e) {
            ajouterErreur($e->getMessage());
            return false;
        }
    }

}

?>